/*
 * Copyright (C) 2011 Greg Dorfuss - mhspot.com
 * 
 * SipToSis is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SipToSis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this source code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * 
 * Author(s):
 * Greg Dorfuss
 */

package local.ua.sscodecs;

import java.util.Arrays;

public class SSCodec_L16_16 implements SSCodec
{
	private String codecName="L16";
	private int payLoadType=-1; // dynamic type
	private int sample_rate=16000;  
	private int sample_size=1;
	private int frame_size=0;
	private int pcmFrameSize;
	private double encodeGain=0;
	private double decodeGain=0;
	private SSCodecUtil codecUtil=new SSCodecUtil();
	
    public void init(int arg_frame_size,double argEncodeGain,double argDecodeGain)
    {
		this.frame_size=arg_frame_size*4;
		this.pcmFrameSize=arg_frame_size*4;
		this.encodeGain=argEncodeGain;
		this.decodeGain=argDecodeGain;
    }
    
    public SSCodecInfo getCodecInfo()
    {
    	return new SSCodecInfo(this.getClass().getSimpleName().replaceAll("^[^_]+_", ""),this.codecName,this.frame_size,this.encodeGain,this.decodeGain);
    }
    
    public int[] getValidFrameSizes()
    {
    	return new int[]{160,240};
    }

	public int getFrameSize()
	{
	   return frame_size;	
	}
	
	public int getPcmFrameSize()
	{
		   return pcmFrameSize;	
	}
	
	public int getSampleRate()
	{
	   return sample_rate;	
	}
	
	public void setSampleRate(int rate)
	{
		sample_rate=rate;
	}
	
	public void setPayloadType(int type)
	{
		payLoadType=type;
	}

	public int getPayloadType()
	{
		return payLoadType;
	}
	public String getCodecName()
	{
		return codecName;
	}

    public int PcmToCodec(byte[] inpcm,int len,byte[] outpcm,int offset)
    {
    	if (encodeGain!=1)
    		codecUtil.changeAmplitude(inpcm,len,inpcm,encodeGain);	
		codecUtil.swapOrder(inpcm,0,outpcm,offset,len);	
    	return len;
    }
    
    final public int CodecToPcm(byte[] inpcm,int offset,int len,byte[] outpcm)
    {
		codecUtil.swapOrder(inpcm,offset,outpcm,0,len);	
    	if (decodeGain!=1)
    		codecUtil.changeAmplitude(outpcm,len,outpcm,decodeGain);	
    	return len;
    }

    public int getCodecBlankPacket(byte[] buf,int offset)
    {
    	Arrays.fill(buf, offset, offset+frame_size,(byte)0);
    	return frame_size;
    }


    public String getCodecXtraMediaAttrib()
    {
    	return null;
    }

	
}
